@@ -21,7 +21,6 @@ use warnings;
 
 our $VERSION = '1.01';
 
-use POSIX qw(:signal_h :sys_wait_h);
 use Carp;
 
 use Dpkg::Compression;
@@ -29,7 +28,7 @@ use Dpkg::Compression::Process;
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
 
-use parent qw(FileHandle Tie::Handle);
+use parent qw(IO::File Tie::Handle);
 
 # Useful reference to understand some kludges required to
 # have the object behave like a filehandle
@@ -103,7 +102,7 @@ and you can't seek on a pipe.
 
 =head1 FileHandle METHODS
 
-The object inherits from FileHandle so all methods that work on this
+The object inherits from IO::File so all methods that work on this
 object should work for Dpkg::Compression::FileHandle too. There
 may be exceptions though.
 
@@ -126,10 +125,10 @@ obviously incompatible with automatic detection of the compression method.
 sub new {
     my ($this, %args) = @_;
     my $class = ref($this) || $this;
-    my $self = FileHandle->new();
+    my $self = IO::File->new();
     # Tying is required to overload the open functions and to auto-open
     # the file on first read/write operation
-    tie *$self, $class, $self;
+    tie *$self, $class, $self; ## no critic (Miscellanea::ProhibitTies)
     bless $self, $class;
     # Initializations
     *$self->{compression} = 'auto';
@@ -432,7 +431,9 @@ sub _cleanup {
     my $cmdline = *$self->{compressor}{cmdline} // '';
     *$self->{compressor}->wait_end_process(nocheck => *$self->{allow_sigpipe});
     if (*$self->{allow_sigpipe}) {
-        unless (($? == 0) || (WIFSIGNALED($?) && (WTERMSIG($?) == SIGPIPE))) {
+        require POSIX;
+        unless (($? == 0) || (POSIX::WIFSIGNALED($?) &&
+                              (POSIX::WTERMSIG($?) == POSIX::SIGPIPE()))) {
             subprocerr($cmdline);
         }
 	*$self->{allow_sigpipe} = 0;
