@@ -101,26 +101,26 @@ sub new {
 }
 
 sub get_arch {
-    my ($self) = @_;
+    my $self = shift;
     return $self->{arch};
 }
 
 sub clear {
-    my ($self) = @_;
+    my $self = shift;
     $self->{objects} = {};
 }
 
 sub clear_except {
     my ($self, @ids) = @_;
-    my %has;
-    $has{$_} = 1 foreach (@ids);
+
+    my %has = map { $_ => 1 } @ids;
     foreach my $objid (keys %{$self->{objects}}) {
 	delete $self->{objects}{$objid} unless exists $has{$objid};
     }
 }
 
 sub get_sonames {
-    my ($self) = @_;
+    my $self = shift;
     return keys %{$self->{objects}};
 }
 
@@ -199,71 +199,75 @@ sub _new_symbol {
     return (ref $base) ? $base->clone(@_) : $base->new(@_);
 }
 
-# Parameter seen is only used for recursive calls
+# Option state is only used for recursive calls.
 sub parse {
-    my ($self, $fh, $file, $seen, $obj_ref, $base_symbol) = @_;
+    my ($self, $fh, $file, %opts) = @_;
+    my $state = $opts{state} //= {};
 
-    if (defined($seen)) {
-	return if exists $seen->{$file}; # Avoid include loops
+    if (defined $state) {
+	return if exists $state->{seen}{$file}; # Avoid include loops
     } else {
 	$self->{file} = $file;
-	$seen = {};
+        $state->{seen} = {};
     }
-    $seen->{$file} = 1;
+    $state->{seen}{$file} = 1;
 
-    if (not ref($obj_ref)) { # Init ref to name of current object/lib
-        $$obj_ref = undef;
+    if (not ref $state->{obj_ref}) { # Init ref to name of current object/lib
+        ${$state->{obj_ref}} = undef;
     }
 
     while (<$fh>) {
 	chomp;
 
 	if (/^(?:\s+|#(?:DEPRECATED|MISSING): ([^#]+)#\s*)(.*)/) {
-	    if (not defined ($$obj_ref)) {
-		error(_g('symbol information must be preceded by a header (file %s, line %s)'), $file, $.);
+	    if (not defined ${$state->{obj_ref}}) {
+		error(g_('symbol information must be preceded by a header (file %s, line %s)'), $file, $.);
 	    }
 	    # Symbol specification
 	    my $deprecated = ($1) ? $1 : 0;
-	    my $sym = _new_symbol($base_symbol, deprecated => $deprecated);
+	    my $sym = _new_symbol($state->{base_symbol}, deprecated => $deprecated);
 	    if ($self->create_symbol($2, base => $sym)) {
-		$self->add_symbol($sym, $$obj_ref);
+		$self->add_symbol($sym, ${$state->{obj_ref}});
 	    } else {
-		warning(_g('failed to parse line in %s: %s'), $file, $_);
+		warning(g_('failed to parse line in %s: %s'), $file, $_);
 	    }
 	} elsif (/^(\(.*\))?#include\s+"([^"]+)"/) {
 	    my $tagspec = $1;
 	    my $filename = $2;
 	    my $dir = $file;
+	    my $old_base_symbol = $state->{base_symbol};
 	    my $new_base_symbol;
 	    if (defined $tagspec) {
-                $new_base_symbol = _new_symbol($base_symbol);
+		$new_base_symbol = _new_symbol($old_base_symbol);
 		$new_base_symbol->parse_tagspec($tagspec);
 	    }
+	    $state->{base_symbol} = $new_base_symbol;
 	    $dir =~ s{[^/]+$}{}; # Strip filename
-	    $self->load("$dir$filename", $seen, $obj_ref, $new_base_symbol);
+	    $self->load("$dir$filename", %opts);
+	    $state->{base_symbol} = $old_base_symbol;
 	} elsif (/^#|^$/) {
 	    # Skip possible comments and empty lines
 	} elsif (/^\|\s*(.*)$/) {
 	    # Alternative dependency template
-	    push @{$self->{objects}{$$obj_ref}{deps}}, "$1";
+	    push @{$self->{objects}{${$state->{obj_ref}}}{deps}}, "$1";
 	} elsif (/^\*\s*([^:]+):\s*(.*\S)\s*$/) {
 	    # Add meta-fields
-	    $self->{objects}{$$obj_ref}{fields}{field_capitalize($1)} = $2;
+	    $self->{objects}{${$state->{obj_ref}}}{fields}{field_capitalize($1)} = $2;
 	} elsif (/^(\S+)\s+(.*)$/) {
 	    # New object and dependency template
-	    $$obj_ref = $1;
-	    if (exists $self->{objects}{$$obj_ref}) {
+	    ${$state->{obj_ref}} = $1;
+	    if (exists $self->{objects}{${$state->{obj_ref}}}) {
 		# Update/override infos only
-		$self->{objects}{$$obj_ref}{deps} = [ "$2" ];
+		$self->{objects}{${$state->{obj_ref}}}{deps} = [ "$2" ];
 	    } else {
 		# Create a new object
-		$self->create_object($$obj_ref, "$2");
+		$self->create_object(${$state->{obj_ref}}, "$2");
 	    }
 	} else {
-	    warning(_g('failed to parse a line in %s: %s'), $file, $_);
+	    warning(g_('failed to parse a line in %s: %s'), $file, $_);
 	}
     }
-    delete $seen->{$file};
+    delete $state->{seen}{$file};
 }
 
 # Beware: we reuse the data structure of the provided symfile so make
@@ -273,7 +277,7 @@ sub merge_object_from_symfile {
     if (not $self->has_object($objid)) {
         $self->{objects}{$objid} = $src->get_object($objid);
     } else {
-        warning(_g('tried to merge the same object (%s) twice in a symfile'), $objid);
+        warning(g_('tried to merge the same object (%s) twice in a symfile'), $objid);
     }
 }
 
@@ -286,19 +290,25 @@ sub output {
     foreach my $soname (sort $self->get_sonames()) {
 	my @deps = $self->get_dependencies($soname);
 	my $dep_first = shift @deps;
-	$dep_first =~ s/#PACKAGE#/$opts{package}/g if exists $opts{package};
+	if (exists $opts{package} and not $opts{template_mode}) {
+	    $dep_first =~ s/#PACKAGE#/$opts{package}/g;
+	}
 	print { $fh } "$soname $dep_first\n" if defined $fh;
 	$res .= "$soname $dep_first\n" if defined wantarray;
 
 	foreach my $dep_next (@deps) {
-	    $dep_next =~ s/#PACKAGE#/$opts{package}/g if exists $opts{package};
+	    if (exists $opts{package} and not $opts{template_mode}) {
+	        $dep_next =~ s/#PACKAGE#/$opts{package}/g;
+	    }
 	    print { $fh } "| $dep_next\n" if defined $fh;
 	    $res .= "| $dep_next\n" if defined wantarray;
 	}
 	my $f = $self->{objects}{$soname}{fields};
 	foreach my $field (sort keys %{$f}) {
 	    my $value = $f->{$field};
-	    $value =~ s/#PACKAGE#/$opts{package}/g if exists $opts{package};
+	    if (exists $opts{package} and not $opts{template_mode}) {
+	        $value =~ s/#PACKAGE#/$opts{package}/g;
+	    }
 	    print { $fh } "* $field: $value\n" if defined $fh;
 	    $res .= "* $field: $value\n" if defined wantarray;
 	}
@@ -360,7 +370,7 @@ sub find_matching_pattern {
 		if (my $alias = $converter->convert_to_alias($name)) {
 		    if ($alias && exists $aliases->{$alias}) {
 			$pattern = $aliases->{$alias};
-			last if &$pattern_ok($pattern);
+			last if $pattern_ok->($pattern);
 			$pattern = undef; # otherwise not found yet
 		    }
 		}
@@ -370,7 +380,7 @@ sub find_matching_pattern {
 	# Now try generic patterns and use the first that matches
 	if (not defined $pattern) {
 	    for my $p (@{$obj->{patterns}{generic}}) {
-		if (&$pattern_ok($p) && $p->matches_rawname($name)) {
+		if ($pattern_ok->($p) && $p->matches_rawname($name)) {
 		    $pattern = $p;
 		    last;
 		}
@@ -392,13 +402,13 @@ sub merge_symbols {
     my ($self, $object, $minver) = @_;
 
     my $soname = $object->{SONAME};
-    error(_g('cannot merge symbols from objects without SONAME'))
+    error(g_('cannot merge symbols from objects without SONAME'))
         unless $soname;
 
     my %include_groups = ();
     my $groups = $self->get_field($soname, 'Ignore-Blacklist-Groups');
     if (defined $groups) {
-        $include_groups{$_} = 1 foreach (split /\s+/, $groups);
+        $include_groups{$_} = 1 foreach (split ' ', $groups);
     }
 
     my %dynsyms;
@@ -457,7 +467,7 @@ sub merge_symbols {
 }
 
 sub is_empty {
-    my ($self) = @_;
+    my $self = shift;
     return scalar(keys %{$self->{objects}}) ? 0 : 1;
 }
 
