@@ -26,7 +26,7 @@ use POSIX qw(:errno_h :fcntl_h);
 use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
-use Dpkg::File;
+use Dpkg::Lock;
 use Dpkg::Dist::Files;
 
 textdomain('dpkg-dev');
